@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2015  Lars Wirzenius
+# Copyright (C) 2009-2016  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,23 +16,19 @@
 
 import errno
 import hashlib
-import larch
 import logging
 import os
 import random
 import re
 import stat
 import time
+
+import larch
 import tracing
 
 import obnamlib
 
 
-class ToplevelIsFileError(obnamlib.ObnamError):
-
-    msg = 'File at repository root: {filename}'
-
-
 class _OpenClientInfo(object):
 
     def __init__(self, client):
@@ -74,10 +70,8 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         self._setup_file_keys()
 
     @classmethod
-    def setup_hooks(self, hooks):
-        hooks.new('repository-toplevel-init')
-        hooks.new_filter('repository-data')
-        hooks.new('repository-add-client')
+    def setup_hooks(cls, hooks):
+        pass
 
     def get_fs(self):
         return self._real_fs
@@ -285,18 +279,18 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             del self._open_client_infos[client_name]
 
     def client_is_locked(self, client_name):
-        logging.info('Checking if %s is locked' % client_name)
+        logging.info('Checking if %s is locked', client_name)
         client_id = self._get_client_id(client_name)
         client_dir = self._get_client_dir(client_id)
         return self._lockmgr.is_locked(client_dir)
 
     def lock_client(self, client_name):
-        logging.info('Locking client %s' % client_name)
+        logging.info('Locking client %s', client_name)
         self._setup_file_key_cache()
         self._raw_lock_client(client_name)
 
     def unlock_client(self, client_name):
-        logging.info('Unlocking client %s' % client_name)
+        logging.info('Unlocking client %s', client_name)
         self._require_existing_client(client_name)
         self._require_client_lock(client_name)
         self._reset_unused_chunks()
@@ -353,7 +347,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         def find_gens_to_keep():
             keep = []
             for gen_id in self.get_client_generation_ids(client_name):
-                a, gen_number = self._unpack_gen_id(gen_id)
+                _, gen_number = self._unpack_gen_id(gen_id)
                 if gen_number not in remove_gen_nos:
                     keep.append(gen_number)
             return keep
@@ -467,7 +461,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         return client_name, gen_number
 
     def _require_existing_generation(self, generation_id):
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         if generation_id not in self.get_client_generation_ids(client_name):
             raise obnamlib.RepositoryGenerationDoesNotExist(
                 client_name=client_name, gen_id=generation_id)
@@ -547,7 +541,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             client_name=client_name, gen_id=genspec)
 
     def make_generation_spec(self, gen_id):
-        client_name, gen_number = self._unpack_gen_id(gen_id)
+        _, gen_number = self._unpack_gen_id(gen_id)
         return str(gen_number)
 
     def remove_generation(self, gen_id):
@@ -568,6 +562,8 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         open_client_info.client.start_changes(create_tree=False)
         open_client_info.client.remove_generation(gen_number)
 
+        return []  # We handle chunk removal ourselves.
+
     def get_generation_chunk_ids(self, generation_id):
         # This intentionally doesn't construct chunk ids for in-tree
         # data, because that's very slow, as it requires iterating
@@ -611,7 +607,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
 
     def _is_in_tree_chunk_id(self, chunk_id):
         # Is this a chunk id for in-tree data?
-        return type(chunk_id) == tuple
+        return isinstance(chunk_id, tuple)
 
     def _chunk_filename(self, chunk_id):
         return self._chunk_idpath.convert(chunk_id)
@@ -671,6 +667,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         pass
 
     def remove_unused_chunks(self):  # pragma: no cover
+        self._require_chunk_indexes_lock()
         for chunk_id in self._unused_chunks:
             self._remove_chunk(chunk_id)
         self._reset_unused_chunks()
@@ -714,7 +711,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             self._fs, self._node_size, self._upload_queue_size,
             self._lru_size, self)
         self._chunksums = obnamlib.ChecksumTree(
-            self._fs, 'chunksums',  len(self._checksum('')), self._node_size,
+            self._fs, 'chunksums', len(self._checksum('')), self._node_size,
             self._upload_queue_size, self._lru_size, self)
 
     def _chunk_index_dirs_to_lock(self):
@@ -817,8 +814,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             self._chunksums.remove_for_all_clients(checksum, chunk_id)
             self._chunklist.remove(chunk_id)
 
-    def find_chunk_ids_by_content(self, data):
-        checksum = self._checksum(data)
+    def find_chunk_ids_by_token(self, checksum):
         candidates = self._chunksums.find(checksum)
         if candidates:
             return candidates
@@ -878,7 +874,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         self._setup_file_key_cache()
 
     def _require_existing_file(self, generation_id, filename):
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
 
         if generation_id not in self.get_client_generation_ids(client_name):
             raise obnamlib.RepositoryGenerationDoesNotExist(
@@ -900,7 +896,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             return False
 
     def add_file(self, generation_id, filename):
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         self._require_client_lock(client_name)
         client = self._open_client(client_name)
         encoded_metadata = obnamlib.fmt_6.metadata_codec.encode_metadata(
@@ -908,7 +904,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         client.create(filename, encoded_metadata)
 
     def remove_file(self, generation_id, filename):
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         self._require_client_lock(client_name)
         self._flush_file_key_cache()
         client = self._open_client(client_name)
@@ -937,8 +933,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
     def _flush_file_key_cache(self):
         for cache_key, value in self._file_key_cache.items():
             generation_id, filename = cache_key
-            client_name, generation_number = self._unpack_gen_id(
-                generation_id)
+            client_name, _ = self._unpack_gen_id(generation_id)
             dirty, metadata = value
             if dirty:
                 encoded_metadata = \
@@ -963,7 +958,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             self._flush_file_key_cache()
             self._require_existing_file(generation_id, filename)
             self._cache_file_keys_from_storage(generation_id, filename)
-        dirty, metadata = self._file_key_cache[cache_key]
+        _, metadata = self._file_key_cache[cache_key]
 
         if key in self._file_keys:
             value = getattr(metadata, self._file_keys[key])
@@ -972,14 +967,14 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             else:
                 return value or ''
         else:
-            client_name, gen_number = self._unpack_gen_id(generation_id)
+            client_name, _ = self._unpack_gen_id(generation_id)
             raise obnamlib.RepositoryFileKeyNotAllowed(
                 format=self.format,
                 client_name=client_name,
                 key_name=obnamlib.repo_key_name(key))
 
     def set_file_key(self, generation_id, filename, key, value):
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         self._require_client_lock(client_name)
 
         cache_key = self._get_file_key_cache_key(generation_id, filename)
@@ -987,7 +982,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
             self._flush_file_key_cache()
             self._require_existing_file(generation_id, filename)
             self._cache_file_keys_from_storage(generation_id, filename)
-        dirty, metadata = self._file_key_cache[cache_key]
+        _, metadata = self._file_key_cache[cache_key]
 
         if key in self._file_keys:
             setattr(metadata, self._file_keys[key], value)
@@ -1011,7 +1006,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
         # We ignore in-tree data here. A file that has real chunks doesn't
         # have in-tree data, so it should be OK.
         self._require_existing_file(generation_id, filename)
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         self._require_client_lock(client_name)
         client = self._open_client(client_name)
         client.set_file_chunks(filename, [])  # FIXME: current gen only
@@ -1019,7 +1014,7 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
     def append_file_chunk_id(self, generation_id, filename, chunk_id):
         assert not self._is_in_tree_chunk_id(chunk_id)
         self._require_existing_file(generation_id, filename)
-        client_name, gen_number = self._unpack_gen_id(generation_id)
+        client_name, _ = self._unpack_gen_id(generation_id)
         self._require_client_lock(client_name)
         client = self._open_client(client_name)
         client.append_file_chunks(filename, [chunk_id])  # FIXME: curgen only
@@ -1045,6 +1040,8 @@ class RepositoryFormat6(obnamlib.RepositoryInterface):
 
 class CheckBTree(obnamlib.WorkItem):  # pragma: no cover
 
+    settings = None
+
     def __init__(self, fs, dirname, skip_setting):
         self.fs = fs
         self.dirname = dirname
@@ -1056,10 +1053,10 @@ class CheckBTree(obnamlib.WorkItem):  # pragma: no cover
             return
 
         if not self.fs.exists(self.dirname):
-            logging.debug('B-tree %s does not exist, skipping' % self.dirname)
+            logging.debug('B-tree %s does not exist, skipping', self.dirname)
             return
 
-        logging.debug('Checking B-tree %s' % self.dirname)
+        logging.debug('Checking B-tree %s', self.dirname)
         fix = self.settings['fsck-fix']
 
         forest = larch.open_forest(
