@@ -37,6 +37,8 @@ class GAClient(object):
         self._client_keys = GAKeys()
         self._generations = GAGenerationList()
         self._data_is_loaded = False
+        self._dir_cache_size = obnamlib.DEFAULT_DIR_CACHE_BYTES
+        self._dir_bag_size = obnamlib.DEFAULT_DIR_BAG_BYTES
 
     def set_current_time(self, current_time):
         self._current_time = current_time
@@ -44,6 +46,16 @@ class GAClient(object):
     def set_fs(self, fs):
         self._fs = fs
 
+    def set_dir_bag_size(self, size):
+        self._dir_bag_size = size
+        if self._blob_store:
+            self._blob_store.set_max_bag_size(self._dir_bag_size)
+
+    def set_dir_cache_size(self, size):
+        self._dir_cache_size = size
+        if self._blob_store:
+            self._blob_store.set_max_cache_bytes(self._dir_cache_size)
+
     def set_dirname(self, dirname):
         self._dirname = dirname
 
@@ -79,9 +91,8 @@ class GAClient(object):
 
             self._blob_store = obnamlib.BlobStore()
             self._blob_store.set_bag_store(bag_store)
-            self._blob_store.set_max_bag_size(obnamlib.DEFAULT_NODE_SIZE)
-            self._blob_store.set_max_cache_bytes(
-                obnamlib.DEFAULT_DIR_OBJECT_CACHE_BYTES)
+            self._blob_store.set_max_bag_size(self._dir_bag_size)
+            self._blob_store.set_max_cache_bytes(self._dir_cache_size)
         return self._blob_store
 
     def _save_per_client_data(self):
@@ -190,8 +201,22 @@ class GAClient(object):
                 client_name=self._client_name,
                 gen_id=gen_number)
 
+        chunks_in_removed = self.get_generation_chunk_ids(gen_number)
+        chunks_remaining = self._get_chunk_ids_used_by_generations(remaining)
+        unused_chunks = set(chunks_in_removed).difference(chunks_remaining)
+
         self._generations.set_generations(remaining)
 
+        return list(unused_chunks)
+
+    def _get_chunk_ids_used_by_generations(self, generations):
+        chunk_ids = set()
+        for generation in generations:
+            gen_number = generation.get_number()
+            chunk_ids = chunk_ids.union(
+                set(self.get_generation_chunk_ids(gen_number)))
+        return chunk_ids
+
     def get_generation_key(self, gen_number, key):
         self._load_data()
         generation = self._lookup_generation_by_gen_number(gen_number)
@@ -264,10 +289,13 @@ class GAClient(object):
 
     def get_metadata_from_file_keys(self, gen_number, filename):
         self._load_data()
-        self._require_file_exists(gen_number, filename)
-
         generation = self._lookup_generation_by_gen_number(gen_number)
         metadata_obj = generation.get_file_metadata()
+        if not metadata_obj.file_exists(filename):
+            raise obnamlib.RepositoryFileDoesNotExistInGeneration(
+                client_name=self._client_name,
+                genspec=gen_number,
+                filename=filename)
         return metadata_obj.get_metadata_from_file_keys(filename)
 
     def set_file_key(self, gen_number, filename, key, value):
@@ -293,24 +321,35 @@ class GAClient(object):
 
     def get_file_chunk_ids(self, gen_number, filename):
         self._load_data()
-        self._require_file_exists(gen_number, filename)
         generation = self._lookup_generation_by_gen_number(gen_number)
         metadata = generation.get_file_metadata()
-        return metadata.get_file_chunk_ids(filename)
+        result = metadata.get_file_chunk_ids(filename)
+        if result is None:
+            raise obnamlib.RepositoryFileDoesNotExistInGeneration(
+                client_name=self._client_name,
+                genspec=gen_number,
+                filename=filename)
+        return result
 
     def append_file_chunk_id(self, gen_number, filename, chunk_id):
         self._load_data()
-        self._require_file_exists(gen_number, filename)
         generation = self._lookup_generation_by_gen_number(gen_number)
         metadata = generation.get_file_metadata()
-        metadata.append_file_chunk_id(filename, chunk_id)
+        if not metadata.append_file_chunk_id(filename, chunk_id):
+            raise obnamlib.RepositoryFileDoesNotExistInGeneration(
+                client_name=self._client_name,
+                genspec=gen_number,
+                filename=filename)
 
     def clear_file_chunk_ids(self, gen_number, filename):
         self._load_data()
-        self._require_file_exists(gen_number, filename)
         generation = self._lookup_generation_by_gen_number(gen_number)
         metadata = generation.get_file_metadata()
-        metadata.clear_file_chunk_ids(filename)
+        if not metadata.clear_file_chunk_ids(filename):
+            raise obnamlib.RepositoryFileDoesNotExistInGeneration(
+                client_name=self._client_name,
+                genspec=gen_number,
+                filename=filename)
 
     def get_generation_chunk_ids(self, gen_number):
         self._load_data()
@@ -324,10 +363,15 @@ class GAClient(object):
 
     def get_file_children(self, gen_number, filename):
         self._load_data()
-        self._require_file_exists(gen_number, filename)
         generation = self._lookup_generation_by_gen_number(gen_number)
         metadata = generation.get_file_metadata()
-        return metadata.get_file_children(filename)
+        result = metadata.get_file_children(filename)
+        if result is None:
+            raise obnamlib.RepositoryFileDoesNotExistInGeneration(
+                client_name=self._client_name,
+                genspec=gen_number,
+                filename=filename)
+        return result
 
     def _is_direct_child_of(self, child, parent):
         return os.path.dirname(child) == parent and child != parent
@@ -376,7 +420,7 @@ class GAGenerationList(object):
 
     def set_generations(self, generations):
         self._generations = generations
-        self._by_number = dict((gen.get_number, gen) for gen in generations)
+        self._by_number = dict((gen.get_number(), gen) for gen in generations)
 
 
 class GAGeneration(object):
@@ -406,9 +450,6 @@ class GAGeneration(object):
     def set_number(self, new_id):
         self._id = new_id
 
-    def keys(self):
-        return self._keys.keys()
-
     def get_key(self, key, default=None):
         return self._keys.get_key(key, default=default)
 
@@ -472,7 +513,7 @@ class GAFileMetadata(object):
     def file_exists(self, filename):
         if filename in self._added_files:
             return True
-        dir_obj, dir_path, basename = self._get_dir_obj(filename)
+        dir_obj, _, basename = self._get_dir_obj(filename)
         return dir_obj and basename in dir_obj.get_file_basenames()
 
     def _get_dir_obj(self, filename):
@@ -525,7 +566,7 @@ class GAFileMetadata(object):
     def get_file_key(self, filename, key):
         if filename in self._added_files:
             return self._added_files.get_file_key(filename, key)
-        dir_obj, dir_path, basename = self._get_dir_obj(filename)
+        dir_obj, _, basename = self._get_dir_obj(filename)
         if dir_obj:
             return dir_obj.get_file_key(basename, key)
         else:
@@ -536,7 +577,7 @@ class GAFileMetadata(object):
             return self._make_metadata(
                 lambda key: self._added_files.get_file_key(filename, key))
         else:
-            dir_obj, dir_path, basename = self._get_dir_obj(filename)
+            dir_obj, _, basename = self._get_dir_obj(filename)
             # We've already verifed the file exists, so we don't need
             # to handle the case where dir_obj is None.
             assert dir_obj is not None
@@ -640,29 +681,33 @@ class GAFileMetadata(object):
             chunk_ids = self._added_files.get_file_chunk_ids(filename)
             return chunk_ids
 
-        dir_obj, dir_path, basename = self._get_dir_obj(filename)
+        dir_obj, _, basename = self._get_dir_obj(filename)
         if dir_obj:
             chunk_ids = dir_obj.get_file_chunk_ids(basename)
             return chunk_ids
         else:
-            return []
+            return None
 
     def append_file_chunk_id(self, filename, chunk_id):
         if filename in self._added_files:
             self._added_files.append_file_chunk_id(filename, chunk_id)
-            return
+            return True
         dir_obj, basename = self._get_mutable_dir_obj(filename)
         if dir_obj:
             dir_obj.append_file_chunk_id(basename, chunk_id)
+            return True
+        return False
 
     def clear_file_chunk_ids(self, filename):
         if filename in self._added_files:
             self._added_files.clear_file_chunk_ids(filename)
-            return
+            return True
         dir_obj, basename = self._get_mutable_dir_obj(filename)
         assert basename != '.'
         if dir_obj:
             dir_obj.clear_file_chunk_ids(basename)
+            return True
+        return False
 
     def get_file_children(self, filename):
         assert filename not in self._added_files
@@ -673,7 +718,7 @@ class GAFileMetadata(object):
             files = [x for x in dir_obj.get_file_basenames() if x != '.']
             subdirs = dir_obj.get_subdir_basenames()
             return [os.path.join(dirname, x) for x in files + subdirs]
-        return []
+        return None
 
 
 class AddedFiles(object):
