@@ -16,6 +16,7 @@
 # =*= License: GPL-3+ =*=
 
 
+import shutil
 import tempfile
 import time
 
@@ -29,10 +30,15 @@ class RepositoryFormatGATests(obnamlib.RepositoryInterfaceTests):
         fs = obnamlib.LocalFS(self.tempdir)
         self.hooks = obnamlib.HookManager()
 
-        # FIXME: The following must be format 6, for now.
-        obnamlib.RepositoryFormat6.setup_hooks(self.hooks)
+        repo_factory = obnamlib.RepositoryFactory()
+        repo_factory.setup_hooks(self.hooks)
 
         self.repo = obnamlib.RepositoryFormatGA(
             hooks=self.hooks,
-            current_time=time.time)
+            current_time=time.time,
+            dir_bag_size=1,
+            dir_cache_size=0)
         self.repo.set_fs(fs)
+
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
