@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2015  Lars Wirzenius
+# Copyright (C) 2009-2016  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@ import logging
 import os
 import re
 import stat
-import time
 import traceback
-import tracing
 import urlparse
 
-import obnamlib
 import larch
+import tracing
+
+import obnamlib
 
 
 class RepositorySettingMissingError(obnamlib.ObnamError):
@@ -51,142 +51,6 @@ class BackupErrors(obnamlib.ObnamError):
     msg = 'There were errors during the backup'
 
 
-class BackupProgress(object):
-
-    def __init__(self, ts):
-        self.file_count = 0
-        self.backed_up_count = 0
-        self.uploaded_bytes = 0
-        self.scanned_bytes = 0
-        self.started = None
-        self.errors = False
-
-        self._ts = ts
-        self._ts['current-file'] = ''
-        self._ts['scanned-bytes'] = 0
-        self._ts['uploaded-bytes'] = 0
-        self._ts.format('%ElapsedTime() '
-                        '%Counter(current-file) '
-                        'files '
-                        '%ByteSize(scanned-bytes) scanned: '
-                        '%String(what)')
-
-    def clear(self):
-        self._ts.clear()
-
-    def error(self, msg, exc=None):
-        self.errors = True
-
-        logging.error(msg)
-        if exc:
-            logging.error(repr(exc))
-        self._ts.error('ERROR: %s' % msg)
-
-    def what(self, what_what):
-        if self.started is None:
-            self.started = time.time()
-        self._ts['what'] = what_what
-        self._ts.flush()
-
-    def update_progress(self):
-        self._ts['not-shown'] = 'not shown'
-
-    def update_progress_with_file(self, filename, metadata):
-        self._ts['what'] = filename
-        self._ts['current-file'] = filename
-        self.file_count += 1
-
-    def update_progress_with_scanned(self, amount):
-        self.scanned_bytes += amount
-        self._ts['scanned-bytes'] = self.scanned_bytes
-
-    def update_progress_with_upload(self, amount):
-        self.uploaded_bytes += amount
-        self._ts['uploaded-bytes'] = self.uploaded_bytes
-
-    def update_progress_with_removed_checkpoint(self, gen):
-        self._ts['checkpoint'] = gen
-
-    def report_stats(self, fs):
-        duration = time.time() - self.started
-        duration_string = obnamlib.humanise_duration(duration)
-
-        chunk_amount, chunk_unit = obnamlib.humanise_size(
-            self.uploaded_bytes)
-
-        ul_amount, ul_unit = obnamlib.humanise_size(fs.bytes_written)
-
-        dl_amount, dl_unit = obnamlib.humanise_size(fs.bytes_read)
-
-        overhead_bytes = (
-            fs.bytes_read + (fs.bytes_written - self.uploaded_bytes))
-        overhead_bytes = max(0, overhead_bytes)
-        overhead_amount, overhead_unit = obnamlib.humanise_size(
-            overhead_bytes)
-        if fs.bytes_written > 0:
-            overhead_percent = 100.0 * overhead_bytes / fs.bytes_written
-        else:
-            overhead_percent = 0.0
-
-        speed_amount, speed_unit = obnamlib.humanise_speed(
-            self.uploaded_bytes, duration)
-
-        logging.info(
-            'Backup performance statistics:')
-        logging.info(
-            '* files found: %s',
-            self.file_count)
-        logging.info(
-            '* files backed up: %s',
-            self.backed_up_count)
-        logging.info(
-            '* uploaded chunk data: %s bytes (%s %s)',
-            self.uploaded_bytes, chunk_amount, chunk_unit)
-        logging.info(
-            '* total uploaded data (incl. metadata): %s bytes (%s %s)',
-            fs.bytes_written, ul_amount, ul_unit)
-        logging.info(
-            '* total downloaded data (incl. metadata): %s bytes (%s %s)',
-            fs.bytes_read, dl_amount, dl_unit)
-        logging.info(
-            '* transfer overhead: %s bytes (%s %s)',
-            overhead_bytes, overhead_amount, overhead_unit)
-        logging.info(
-            '* duration: %s s (%s)',
-            duration, duration_string)
-        logging.info(
-            '* average speed: %s %s',
-            speed_amount, speed_unit)
-
-        scanned_amount, scanned_unit = obnamlib.humanise_size(
-            self.scanned_bytes)
-
-        self._ts.notify(
-            'Backed up %d files (of %d found), containing %.1f %s.' %
-            (self.backed_up_count,
-             self.file_count,
-             scanned_amount,
-             scanned_unit))
-        self._ts.notify(
-            'Uploaded %.1f %s file data in %s at %.1f %s average speed.' %
-            (chunk_amount,
-             chunk_unit,
-             duration_string,
-             speed_amount,
-             speed_unit))
-        self._ts.notify(
-            'Total download amount %.1f %s.' %
-            (dl_amount,
-             dl_unit))
-        self._ts.notify(
-            'Total upload amount %.1f %s. Overhead was %.1f %s (%.1f %%).' %
-            (ul_amount,
-             ul_unit,
-             overhead_amount,
-             overhead_unit,
-             overhead_percent))
-
-
 class CheckpointManager(object):
 
     def __init__(self, repo, checkpoint_interval):
@@ -256,10 +120,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
 
         self.app.settings.boolean(
             ['small-files-in-btree'],
-            'put contents of small files directly into '
-            'the per-client B-tree, instead of '
-            'separate chunk files; do not use this '
-            'as it is quite bad for performance',
+            'this is available only for backwards compatibility; '
+            'do not use it, and remove it from your configuration',
             group=backup_group)
 
         # Performance related settings.
@@ -296,6 +158,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
 
         root_urls = self.app.settings['root'] + args
         self.check_for_required_settings(root_urls)
+        self.pretend = self.app.settings['pretend']
 
         self.start_backup()
         try:
@@ -308,7 +171,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                     self.remove_checkpoints()
             self.finish_backup(args)
         except BaseException, e:
-            logging.debug('Handling exception %s' % str(e))
+            logging.debug('Handling exception %s', str(e))
             logging.debug(traceback.format_exc())
             self.unlock_when_error()
             raise
@@ -343,13 +206,13 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             self.app.settings['checkpoint'])
 
     def configure_progress_reporting(self):
-        self.progress = BackupProgress(self.app.ts)
+        self.progress = obnamlib.BackupProgress(self.app.ts)
 
     def open_repository(self):
         if self.pretend:
             try:
                 return self.app.get_repository_object()
-            except Exception as e:
+            except Exception:
                 self.progress.error(
                     'Are you using --pretend without an existing '
                     'repository? That does not\n'
@@ -381,10 +244,10 @@ class BackupPlugin(obnamlib.ObnamPlugin):
     def finish_generation(self):
         prefix = 'committing changes to repository: '
 
-        self.progress.what(prefix + 'locking shared B-trees')
+        self.progress.what(prefix + 'locking shared chunk indexes')
         self.repo.lock_chunk_indexes()
 
-        self.progress.what(prefix + 'adding chunks to shared B-trees')
+        self.progress.what(prefix + 'adding chunks to shared chunk indexes')
         self.add_chunks_to_shared()
 
         self.progress.what(prefix + 'updating generation metadata')
@@ -406,7 +269,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.repo.commit_client(self.client_name)
         self.repo.unlock_client(self.client_name)
 
-        self.progress.what(prefix + 'committing shared B-trees')
+        self.progress.what(prefix + 'committing shared chunk indexes')
         self.repo.commit_chunk_indexes()
         self.repo.unlock_chunk_indexes()
 
@@ -428,9 +291,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.repo.commit_client(self.client_name)
         self.repo.unlock_client(self.client_name)
 
-        self.progress.what(prefix + ': commiting shared B-trees')
+        self.progress.what(prefix + ': commiting shared chunk indexes')
         self.repo.commit_chunk_indexes()
-        self.repo.unlock_chunk_indexes()
 
         self.progress.what(prefix + ': removing unused chunks')
         self.repo.remove_unused_chunks()
@@ -442,7 +304,9 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.repo.close()
 
         self.progress.clear()
-        self.progress.report_stats(self.repo.get_fs())
+        self.progress.finish()
+        self.progress.report_stats(
+            self.app.output, self.repo.get_fs(), self.app.settings['quiet'])
 
         logging.info('Backup finished.')
         self.app.hooks.call('backup-finished', args, self.progress)
@@ -453,10 +317,6 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         p.set_default_unit('MiB')
         return p.parse(value)
 
-    @property
-    def pretend(self):
-        return self.app.settings['pretend']
-
     @property
     def client_name(self):
         return self.app.settings['client-name']
@@ -471,8 +331,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                     'Attempting to unlock shared trees because of error')
                 self.repo.unlock_chunk_indexes()
         except BaseException, e2:
-            logging.warning(
-                'Error while unlocking due to error: %s' % str(e2))
+            logging.warning('Error while unlocking due to error: %s', str(e2))
             logging.debug(traceback.format_exc())
         else:
             logging.info('Successfully unlocked')
@@ -483,7 +342,12 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.chunkid_token_map.clear()
 
     def add_client(self, client_name):
-        self.repo.lock_client_list()
+        try:
+            self.repo.lock_client_list()
+        except obnamlib.GpgError:
+            self.repo.unlock_client_list()
+            raise
+
         if client_name not in self.repo.get_client_names():
             tracing.trace('adding new client %s' % client_name)
             tracing.trace('client list before adding: %s' %
@@ -508,7 +372,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             self.fs.close()
 
     def backup_root(self, root, absroots):
-        logging.info('Backing up root %s' % root)
+        logging.info('Backing up root %s', root)
         self.progress.what('connecting to live data %s' % root)
 
         self.reopen_fs(root)
@@ -525,7 +389,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.root_metadata = self.fs.lstat(absroot)
 
         for pathname, metadata in self.find_files(absroot):
-            logging.info('Backing up %s' % pathname)
+            logging.info('Backing up %s', pathname)
             if not self.pretend:
                 existed = self.repo.file_exists(self.new_generation, pathname)
             try:
@@ -540,7 +404,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                 self.progress.error(msg, exc=e)
                 if not existed and not self.pretend:
                     self.remove_partially_backed_up_file(pathname)
-                if e.errno == errno.ENOSPC:
+                if e.errno in (errno.ENOSPC, errno.EPIPE):
                     raise
 
             if self.checkpoint_manager.time_for_checkpoint():
@@ -550,7 +414,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.backup_parents('.')
 
     def translate_enverror_to_obnamerror(self, pathname, exc):
-        if type(exc) is IOError:
+        if isinstance(exc, IOError):
             return obnamlib.ObnamIOError(
                 errno=exc.errno,
                 strerror=exc.strerror,
@@ -579,8 +443,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
 
         self.backup_metadata(pathname, metadata)
         if stat.S_ISREG(metadata.st_mode):
-            assert metadata.md5 is None
-            metadata.md5 = self.backup_file_contents(pathname, metadata)
+            self.backup_file_contents(pathname, metadata)
         self.backup_metadata(pathname, metadata)
 
     def open_fs(self, root_url):
@@ -593,16 +456,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.open_or_reopen_fs(self.fs.reinit, root_url)
 
     def open_or_reopen_fs(self, func, root_url):
-        scheme, netloc, path, params, query, fragment = \
-            urlparse.urlparse(root_url)
-
         try:
             func(root_url)
-            if not self.fs.isdir(path):
-                parent_path = os.path.dirname(path)
-                parent_url = urlparse.urlunparse(
-                    (scheme, netloc, parent_path, params, query, fragment))
-                self.fs.reinit(parent_url)
         except OSError as e:
             if e.errno == errno.ENOENT:
                 raise BackupRootDoesNotExist(root=root_url)
@@ -613,7 +468,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         for root_url in root_urls:
             self.progress.what('determining absolute path for %s' % root_url)
 
-            scheme, neloc, path, params, query, fragment = \
+            scheme, netloc, path, params, query, fragment = \
                 urlparse.urlparse(root_url)
 
             if self.fs.isdir(path):
@@ -637,7 +492,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             # details of the old repository class. Should
             # be cleaned up, someday.
             pass
-        except Exception as ee:
+        except (OSError, IOError, obnamlib.ObnamError) as ee:
             msg = (
                 'Error removing partly backed up file %s: %s'
                 % (pathname, repr(ee)))
@@ -660,15 +515,16 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             self.progress.what('making checkpoint: backing up parents')
             self.backup_parents('.')
 
-            self.progress.what('making checkpoint: locking shared B-trees')
+            self.progress.what(
+                'making checkpoint: locking shared chunk indexes')
             self.repo.lock_chunk_indexes()
 
             self.progress.what(
-                'making checkpoint: adding chunks to shared B-trees')
+                'making checkpoint: adding chunks to shared chunk indexes')
             self.add_chunks_to_shared()
 
             self.progress.what(
-                'making checkpoint: committing per-client B-tree')
+                'making checkpoint: committing per-client data')
             self.repo.set_generation_key(
                 self.new_generation,
                 obnamlib.REPO_GENERATION_IS_CHECKPOINT, 1)
@@ -676,7 +532,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             self.repo.commit_client(self.client_name)
             self.repo.unlock_client(self.client_name)
 
-            self.progress.what('making checkpoint: committing shared B-trees')
+            self.progress.what(
+                'making checkpoint: committing shared chunk indexes')
             self.repo.commit_chunk_indexes()
             self.repo.unlock_chunk_indexes()
             self.last_checkpoint = self.repo.get_fs().bytes_written
@@ -729,6 +586,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         exclude = [False]
         self.app.hooks.call(
             'backup-exclude',
+            progress=self.progress,
             fs=self.fs,
             pathname=pathname,
             stat_result=st,
@@ -776,7 +634,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
 
         try:
             old = self.get_metadata_from_generation(gen, pathname)
-        except obnamlib.ObnamError as e:
+        except obnamlib.ObnamError:
             # File does not exist in the previous generation, so it
             # does need to be backed up.
             return True
@@ -828,9 +686,9 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                 metadata = obnamlib.read_metadata(self.fs, root)
             except OSError, e:
                 logging.warning(
-                    'Failed to get metadata for %s: %s: %s' %
-                    (root, e.errno or 0, e.strerror))
-                logging.warning('Using fake metadata instead for %s' % root)
+                    'Failed to get metadata for %s: %s: %s',
+                    root, e.errno or 0, e.strerror)
+                logging.warning('Using fake metadata instead for %s', root)
                 metadata = dummy_metadata
             if not self.pretend:
                 self.add_file_to_generation(root, metadata)
@@ -849,7 +707,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
     def backup_file_contents(self, filename, metadata):
         '''Back up contents of a regular file.
 
-        Return MD5 checksum of file's complete data.
+        Sets the whole-file MD5 checksum in metadata as a side effect.
 
         '''
 
@@ -869,7 +727,10 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         tracing.trace('opening file for reading')
         f = self.fs.open(filename, 'r')
 
-        summer = hashlib.md5()
+        if obnamlib.REPO_FILE_MD5 in self.repo.get_allowed_file_keys():
+            summer = hashlib.md5()
+        else:
+            summer = None
 
         chunk_size = int(self.app.settings['chunk-size'])
         while True:
@@ -881,7 +742,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                 break
             tracing.trace('got %d bytes of data' % len(data))
             self.progress.update_progress_with_scanned(len(data))
-            summer.update(data)
+            if summer:
+                summer.update(data)
             if not self.pretend:
                 chunk_id = self.backup_file_chunk(data)
                 self.repo.append_file_chunk_id(
@@ -900,7 +762,8 @@ class BackupPlugin(obnamlib.ObnamPlugin):
         self.app.dump_memory_profile('at end of file content backup for %s' %
                                      filename)
         tracing.trace('done backing up file contents')
-        return summer.digest()
+        if summer:
+            metadata.md5 = summer.digest()
 
     def backup_file_chunk(self, data):
         '''Back up a chunk of data by putting it into the repository.'''
@@ -912,7 +775,7 @@ class BackupPlugin(obnamlib.ObnamPlugin):
             # exceptions, and other errors. We don't care: we'll just
             # pretend no chunk with the checksum exists yet.
             try:
-                in_tree = self.repo.find_chunk_ids_by_content(data)
+                in_tree = self.repo.find_chunk_ids_by_token(token)
             except larch.Error:
                 in_tree = []
             except obnamlib.RepositoryChunkContentNotInIndexes:
@@ -940,10 +803,9 @@ class BackupPlugin(obnamlib.ObnamPlugin):
                 if data == data2:
                     share(chunkid)
                     return chunkid
-            else:
-                chunkid = put()
-                share(chunkid)
-                return chunkid
+            chunkid = put()
+            share(chunkid)
+            return chunkid
         elif mode == 'fatalist':
             existing = find()
             if existing:
