@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2015  Lars Wirzenius <liw@liw.fi>
+# Copyright (C) 2009-2016  Lars Wirzenius <liw@liw.fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,16 +28,7 @@ import time
 import urlparse
 import getpass
 
-
-# As of 2010-07-10, Debian's paramiko package triggers
-# RandomPool_DeprecationWarning. This will eventually be fixed. Until
-# then, there is no point in spewing the warning to the user, who can't
-# do nothing.
-# http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=586925
-import warnings
-with warnings.catch_warnings():
-    warnings.simplefilter('ignore')
-    import paramiko
+import paramiko
 
 import obnamlib
 
@@ -89,6 +80,11 @@ class HardlinkError(obnamlib.ObnamError):
     '''
 
 
+class UnknownParamikoVersion(obnamlib.ObnamError):
+
+    msg = '''Cannot find out version of Paramiko.'''
+
+
 def ioerror_to_oserror(method):
     '''Decorator to convert an IOError exception to OSError.
 
@@ -176,11 +172,11 @@ class SftpFS(obnamlib.VirtualFileSystem):
 
     def log_stats(self):
         obnamlib.VirtualFileSystem.log_stats(self)
-        logging.info('VFS: baseurl=%s roundtrips=%s' %
-                     (self.baseurl, self._roundtrips))
+        logging.info(
+            'VFS: baseurl=%s roundtrips=%s', self.baseurl, self._roundtrips)
 
     def _to_string(self, str_or_unicode):
-        if type(str_or_unicode) is unicode:
+        if isinstance(str_or_unicode, unicode):
             return str_or_unicode.encode('utf-8')
         else:
             return str_or_unicode
@@ -188,7 +184,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
     def _create_root_if_missing(self):
         try:
             self.mkdir(self.path)
-        except OSError, e:
+        except OSError:
             # sftp/paramiko does not give us a useful errno so we hope
             # for the best
             pass
@@ -230,7 +226,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
 
         # prepend the executable to the argument list
         args.insert(0, executable)
-        logging.debug('executing openssh: %s' % args)
+        logging.debug('executing openssh: %s', args)
         try:
             proc = subprocess.Popen(args,
                                     stdin=subprocess.PIPE,
@@ -245,14 +241,13 @@ class SftpFS(obnamlib.VirtualFileSystem):
 
     def _connect_paramiko(self):
         remote = (self.host, self.port or 22)
-        logging.debug(
-            'connect_paramiko: host=%s port=%s' % remote)
+        logging.debug('connect_paramiko: host=%s port=%s', *remote)
         self.transport = paramiko.Transport(remote)
         self.transport.connect()
         logging.debug('connect_paramiko: connected')
         try:
             self._check_host_key(self.host)
-        except BaseException, e:
+        except BaseException:
             self.transport.close()
             self.transport = None
             raise
@@ -263,7 +258,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         logging.debug('connect_paramiko: end')
 
     def _check_host_key(self, hostname):
-        logging.debug('checking ssh host key for %s' % hostname)
+        logging.debug('checking ssh host key for %s', hostname)
 
         offered_key = self.transport.get_remote_server_key()
 
@@ -274,8 +269,8 @@ class SftpFS(obnamlib.VirtualFileSystem):
         if known_keys is None:
             if self.settings['ssh-host-keys-check'] == 'yes':
                 raise NoHostKeyError(hostname=hostname)
-            logging.warning('No known host keys for %s; accepting offered key'
-                            % hostname)
+            logging.warning(
+                'No known host keys for %s; accepting offered key', hostname)
             return
 
         offered_type = offered_key.get_name()
@@ -283,14 +278,15 @@ class SftpFS(obnamlib.VirtualFileSystem):
             if self.settings['ssh-host-keys-check'] == 'yes':
                 raise NoHostKeyOfWantedTypeError(
                     key_type=offered_type, hostname=hostname)
-            logging.warning('No known host key of type %s for %s; accepting '
-                            'offered key' % (offered_type, hostname))
+            logging.warning(
+                'No known host key of type %s for %s; accepting offered key',
+                offered_type, hostname)
 
         known_key = known_keys[offered_type]
         if offered_key != known_key:
             raise WrongHostKeyError(hostname=hostname)
 
-        logging.debug('Host key for %s OK' % hostname)
+        logging.debug('Host key for %s OK', hostname)
 
     def _authenticate(self, username):
         if not username:
@@ -334,38 +330,17 @@ class SftpFS(obnamlib.VirtualFileSystem):
 
     @ioerror_to_oserror
     def reinit(self, baseurl, create=False):
-        scheme, netloc, path, query, fragment = urlparse.urlsplit(baseurl)
+        p = scheme, _, path, _, _ = urlparse.urlsplit(baseurl)
 
         if scheme != 'sftp':
             raise WrongURLSchemeError(url=baseurl)
 
-        if '@' in netloc:
-            user, netloc = netloc.split('@', 1)
-        else:
-            user = None
-
-        if ':' in netloc:
-            host, port = netloc.split(':', 1)
-            if port == '':
-                port = None
-            else:
-                try:
-                    port = int(port)
-                except ValueError, e:
-                    exc = InvalidPortError(
-                        port=port, url=baseurl, error=str(e))
-                    logging.error(str(exc))
-                    raise exc
-        else:
-            host = netloc
-            port = None
-
         if path.startswith('/~/'):
             path = path[3:]
 
-        self.host = host
-        self.port = port
-        self.user = user
+        self.host = p.hostname
+        self.port = p.port
+        self.user = p.username
         self.path = path
         self.create_path_if_missing = create
 
@@ -374,7 +349,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         if self.sftp:
             if create:
                 self._create_root_if_missing()
-            logging.debug('chdir to %s' % path)
+            logging.debug('chdir to %s', path)
             self.sftp.chdir(self._initial_dir)
             self.sftp.chdir(path)
 
@@ -536,7 +511,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
     def lchown(self, pathname, uid, gid):
         self._delay()
         if stat.S_ISLNK(self.lstat(pathname).st_mode):
-            logging.warning('NOT changing ownership of symlink %s' % pathname)
+            logging.warning('NOT changing ownership of symlink %s', pathname)
         else:
             self.sftp.chown(pathname, uid, gid)
 
@@ -585,7 +560,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
     def cat(self, pathname):
         self._delay()
         f = self.open(pathname, 'rb')
-        f.prefetch()
+        self._prefetch(pathname, f)
         chunks = []
         while True:
             chunk = f.read(self.chunk_size)
@@ -596,6 +571,41 @@ class SftpFS(obnamlib.VirtualFileSystem):
         f.close()
         return ''.join(chunks)
 
+    def _prefetch(self, pathname, f):
+        '''Call f.prefetch in the right way.
+
+        Up to paramiko version 1.15.1, prefetch accepts no arguments.
+        In 1.16 it requires one.
+
+        '''
+
+        version_info = self._get_paramiko_version_info()
+        if version_info < (1, 16):
+            f.prefetch()
+        else:
+            file_size = self.lstat(pathname).st_size
+            f.prefetch(file_size)
+
+    def _get_paramiko_version_info(self):
+        '''Get the __version_info__ tuple for paramiko.
+
+        Some versions of paramiko (1.7.8 to 1.10.4, according to
+        research by Jan Niggemann) have no __version_info__ tuple. We
+        need that tuple to do version comparisons.
+
+        This method constructs the tuple from paramiko.__version__, if
+        paramiko.__version_info__ is missing. If __version__ is also
+        missing, it raises an exception.
+
+        '''
+
+        if hasattr(paramiko, '__version_info__'):
+            return paramiko.__version_info__
+        elif hasattr(paramiko, '__version__'):
+            return tuple(int(x) for x in paramiko.__version__.split('.'))
+        else:
+            raise UnknownParamikoVersion()
+
     @ioerror_to_oserror
     def write_file(self, pathname, contents):
         mode = 'wbx'
@@ -618,6 +628,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         f.close()
 
     def _tempfile(self, dirname):
+
         '''Create a new file with a random name, return handle and name.'''
 
         if dirname:
@@ -653,7 +664,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
                 # on creation, so we set it separately. This leaves a
                 # short window where the file is possible to open.
                 self.chmod_not_symlink(pathname, obnamlib.NEW_FILE_MODE)
-            except (IOError, OSError) as e:
+            except (IOError, OSError):
                 if try_number == max_tries - 1:
                     raise
             else:
