@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2015  Lars Wirzenius
+# Copyright (C) 2009-2016  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -147,8 +147,6 @@ class ShowPlugin(obnamlib.ObnamPlugin):
         for gen_id in self.repo.get_client_generation_ids(client_name):
             start = self.repo.get_generation_key(
                 gen_id, obnamlib.REPO_GENERATION_STARTED)
-            end = self.repo.get_generation_key(
-                gen_id, obnamlib.REPO_GENERATION_ENDED)
             if most_recent is None or start > most_recent:
                 most_recent = start
         self.repo.close()
@@ -211,7 +209,22 @@ class ShowPlugin(obnamlib.ObnamPlugin):
         return filename
 
     def format_time(self, timestamp):
-        return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(timestamp))
+        prefix = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(timestamp))
+        return prefix + ' ' + self.format_timezone()
+
+    def format_timezone(self):
+        '''Return a timezone indication on +0300 format.'''
+
+        # A zero offset gets a plus sign. The time.timezone value has the
+        # opposite sign of what we want in the output.
+        if time.timezone <= 0:
+            sign = '+'
+        else:
+            sign = '-'
+
+        hh = abs(time.timezone) / 3600
+        mm = (abs(time.timezone) % 3600) / 60
+        return '%c%02d%02d' % (sign, hh, mm)
 
     def isdir(self, gen_id, filename):
         mode = self.repo.get_file_key(
@@ -235,7 +248,7 @@ class ShowPlugin(obnamlib.ObnamPlugin):
         self.traverse(self.show_hdr_ls, self.show_item_ls, args)
 
     def show_hdr_ls(self, comment):
-            self.app.output.write(comment)
+        self.app.output.write(comment)
 
     def show_item_ls(self, gen_id, filename):
         fields = self.fields(gen_id, filename)
@@ -250,12 +263,12 @@ class ShowPlugin(obnamlib.ObnamPlugin):
         ]
 
         result = []
-        for i in range(len(fields)):
+        for i, field in enumerate(fields):
             if widths[i] < 0:
                 fmt = '%-*s'
             else:
                 fmt = '%*s'
-            result.append(fmt % (abs(widths[i]), fields[i]))
+            result.append(fmt % (abs(widths[i]), field))
         self.app.output.write('%s\n' % ' '.join(result))
 
     def kdirstat(self, args):
@@ -289,8 +302,13 @@ class ShowPlugin(obnamlib.ObnamPlugin):
             mode_str = "BlockDev\t"
         elif stat.S_ISCHR(mode):
             mode_str = "CharDev\t"
+        elif stat.S_ISFIFO(mode):
+            mode_str = "FIFO"
         elif stat.S_ISSOCK(mode):
             mode_str = "Socket\t"
+        else:
+            # Unhandled, make it look like a comment
+            mode_str = "#UNHANDLED\t"
 
         enc_filename = filename.replace("%", "%25")
         enc_filename = enc_filename.replace(" ", "%20")
@@ -330,8 +348,9 @@ class ShowPlugin(obnamlib.ObnamPlugin):
             # Files are both present and neither is a directory.
             # Compare md5
             def get_md5(gen_id):
-                return self.repo.get_file_key(
-                    gen_id, fullname, obnamlib.REPO_FILE_MD5)
+                if obnamlib.REPO_FILE_MD5 in self.repo.get_allowed_file_keys():
+                    return self.repo.get_file_key(
+                        gen_id, fullname, obnamlib.REPO_FILE_MD5)
             md5_1 = get_md5(gen_id1)
             md5_2 = get_md5(gen_id2)
             if md5_1 != md5_2:
