@@ -1,6 +1,6 @@
 # repo_interface.py -- interface class for repository access
 #
-# Copyright 2013-2015  Lars Wirzenius
+# Copyright 2013-2016  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,48 +38,59 @@ import obnamlib
 # little magic to get a) automatic enumeration b) mapping between
 # values and names.
 
-_string_keys = [
-    "REPO_CLIENT_TEST_KEY",
-    "REPO_GENERATION_TEST_KEY",
-    "REPO_FILE_TEST_KEY",
-    "REPO_FILE_USERNAME",
-    "REPO_FILE_GROUPNAME",
-    "REPO_FILE_SYMLINK_TARGET",
-    "REPO_FILE_XATTR_BLOB",
-    "REPO_FILE_MD5",
-]
 
-_integer_keys = [
-    "REPO_GENERATION_STARTED",
-    "REPO_GENERATION_ENDED",
-    "REPO_GENERATION_IS_CHECKPOINT",
-    "REPO_GENERATION_FILE_COUNT",
-    "REPO_GENERATION_TOTAL_DATA",
+_saved_id = 0
 
-    "REPO_FILE_MODE",
-    "REPO_FILE_MTIME_SEC",
-    "REPO_FILE_MTIME_NSEC",
-    "REPO_FILE_ATIME_SEC",
-    "REPO_FILE_ATIME_NSEC",
-    "REPO_FILE_NLINK",
-    "REPO_FILE_SIZE",
-    "REPO_FILE_UID",
-    "REPO_FILE_GID",
-    "REPO_FILE_BLOCKS",
-    "REPO_FILE_DEV",
-    "REPO_FILE_INO",
-]
 
-_repo_key_names = {}
-for i, name in enumerate(_string_keys + _integer_keys):
-    globals()[name] = i
-    _repo_key_names[i] = name
+def _get_next_id():
+    global _saved_id
+    _saved_id += 1
+    return _saved_id
+
+
+REPO_CLIENT_TEST_KEY = _get_next_id()
+REPO_GENERATION_TEST_KEY = _get_next_id()
+REPO_FILE_TEST_KEY = _get_next_id()
+REPO_FILE_USERNAME = _get_next_id()
+REPO_FILE_GROUPNAME = _get_next_id()
+REPO_FILE_SYMLINK_TARGET = _get_next_id()
+REPO_FILE_XATTR_BLOB = _get_next_id()
+REPO_FILE_MD5 = _get_next_id()
+
+_MAX_STRING_KEY = REPO_FILE_MD5
+
+REPO_GENERATION_STARTED = _get_next_id()
+REPO_GENERATION_ENDED = _get_next_id()
+REPO_GENERATION_IS_CHECKPOINT = _get_next_id()
+REPO_GENERATION_FILE_COUNT = _get_next_id()
+REPO_GENERATION_TOTAL_DATA = _get_next_id()
+REPO_FILE_MODE = _get_next_id()
+REPO_FILE_MTIME_SEC = _get_next_id()
+REPO_FILE_MTIME_NSEC = _get_next_id()
+REPO_FILE_ATIME_SEC = _get_next_id()
+REPO_FILE_ATIME_NSEC = _get_next_id()
+REPO_FILE_NLINK = _get_next_id()
+REPO_FILE_SIZE = _get_next_id()
+REPO_FILE_UID = _get_next_id()
+REPO_FILE_GID = _get_next_id()
+REPO_FILE_BLOCKS = _get_next_id()
+REPO_FILE_DEV = _get_next_id()
+REPO_FILE_INO = _get_next_id()
+
+
+_repo_key_names = dict(
+    (globals()[name], name)
+    for name in globals()
+    if name.startswith('REPO_'))
 
 
 def _filter_integer_keys(prefix):
-    return [globals()[name]
-            for name in _integer_keys
-            if name.startswith(prefix)]
+    return [
+        globals()[name]
+        for name in globals()
+        if name.startswith(prefix) and globals()[name] > _MAX_STRING_KEY
+    ]
+
 
 REPO_GENERATION_INTEGER_KEYS = _filter_integer_keys('REPO_GENERATION_')
 REPO_FILE_INTEGER_KEYS = _filter_integer_keys('REPO_FILE_')
@@ -333,7 +344,7 @@ class RepositoryInterface(object):
     # Operations on the repository itself.
 
     @classmethod
-    def setup_hooks(self, hooks):  # pragma: no cover
+    def setup_hooks(cls, hooks):  # pragma: no cover
         '''Create any hooks for this repository format.
 
         Note that this is a class method.
@@ -393,22 +404,30 @@ class RepositoryInterface(object):
 
         '''
         unlockers = []
+        gpg_unlockers = []
         try:
             if not self.got_client_list_lock():
+                gpg_unlockers.append((self.unlock_client_list, []))
                 self.lock_client_list()
                 unlockers.append((self.unlock_client_list, []))
 
             for client_name in self.get_client_names():
                 if not self.got_client_lock(client_name):
+                    gpg_unlockers.append((self.unlock_client, [client_name]))
                     self.lock_client(client_name)
                     unlockers.append((self.unlock_client, [client_name]))
 
             if not self.got_chunk_indexes_lock():
+                gpg_unlockers.append((self.unlock_chunk_indexes, []))
                 self.lock_chunk_indexes()
         except obnamlib.LockFail:
             for unlocker, args in unlockers:
                 unlocker(*args)
             raise
+        except obnamlib.GpgError:
+            for unlocker, args in gpg_unlockers:
+                unlocker(*args)
+            raise
 
     def unlock_everything(self):
         '''Unock every part of the repository to which we hold a lock.
@@ -625,6 +644,9 @@ class RepositoryInterface(object):
 
         The removed generation may be the currently unfinished one.
 
+        Return a list of chunk ids that are no longer used by this
+        client.
+
         '''
         raise NotImplementedError()
 
@@ -824,7 +846,7 @@ class RepositoryInterface(object):
         '''Write any pending new chunks to repository.'''
         raise NotImplementedError()
 
-    def remove_unused_chunk(self):
+    def remove_unused_chunks(self):
         '''Remove chunks that are no longer used by any client.
 
         The caller MUST commit any changes to clients or chunk indexes
@@ -909,9 +931,11 @@ class RepositoryInterface(object):
         '''Removes a chunk from indexes, given its id, for all clients.'''
         raise NotImplementedError()
 
-    def find_chunk_ids_by_content(self, data):
+    def find_chunk_ids_by_token(self, token):
         '''Finds chunk ids that probably match a given content.
 
+        The token must be a return value of prepare_chunk_for_indexes.
+
         This will raise RepositoryChunkContentNotInIndexes if the
         chunk is not in the indexes. Otherwise it will return all
         chunk ids that would have the same token (see
@@ -970,6 +994,8 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
 
     # Tests for repository level things.
 
+    repo = None
+
     def test_has_format_attribute(self):
         self.assertEqual(type(self.repo.format), str)
 
@@ -980,8 +1006,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
     def test_closes_repository(self):
         # Can't think of a test to verify the closing happened,
         # so just calling the method will have to do for now.
-        self.repo.close()
-        self.assertTrue(True)
+        self.assertEqual(self.repo.close(), None)
 
     def test_returns_list_of_shared_directories(self):
         self.assertTrue(type(self.repo.get_shared_directories()), list)
@@ -1452,7 +1477,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
     def test_unlocking_client_removes_created_generation(self):
         self.setup_client()
         self.repo.lock_client('fooclient')
-        new_id = self.repo.create_generation('fooclient')
+        self.repo.create_generation('fooclient')
         self.repo.unlock_client('fooclient')
         self.assertEqual(self.repo.get_client_generation_ids('fooclient'), [])
 
@@ -1490,7 +1515,9 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         gen_id = self.create_generation()
         for key in self.repo.get_allowed_generation_keys():
             value = self.repo.get_generation_key(gen_id, key)
-            self.assertTrue(type(value) in (str, int))
+            self.assertTrue(
+                isinstance(value, str) or
+                isinstance(value, int))
 
     def test_has_empty_string_for_generation_test_key(self):
         if self.generation_test_key_is_allowed():
@@ -1538,16 +1565,18 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
 
     def test_removes_unfinished_generation(self):
         gen_id = self.create_generation()
-        self.repo.remove_generation(gen_id)
+        chunk_ids = self.repo.remove_generation(gen_id)
         self.assertEqual(self.repo.get_client_generation_ids('fooclient'), [])
+        self.assertEqual(type(chunk_ids), list)
 
     def test_removes_finished_generation(self):
         gen_id = self.create_generation()
         self.repo.commit_client('fooclient')
         self.repo.unlock_client('fooclient')
         self.repo.lock_client('fooclient')
-        self.repo.remove_generation(gen_id)
+        chunk_ids = self.repo.remove_generation(gen_id)
         self.assertEqual(self.repo.get_client_generation_ids('fooclient'), [])
+        self.assertEqual(type(chunk_ids), list)
 
     def test_removing_removed_generation_fails(self):
         gen_id = self.create_generation()
@@ -1612,7 +1641,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.commit_client('fooclient')
         self.repo.unlock_client('fooclient')
         self.repo.lock_client('fooclient')
-        gen_id_2 = self.repo.create_generation('fooclient')
+        self.repo.create_generation('fooclient')
         genspec = self.repo.make_generation_spec(gen_id)
         self.repo.remove_generation(gen_id)
         self.assertRaises(
@@ -1672,7 +1701,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.unlock_client('fooclient')
 
         self.repo.lock_client('fooclient')
-        gen_id_2 = self.repo.create_generation('fooclient')
+        self.repo.create_generation('fooclient')
         self.repo.remove_file(gen_id, '/foo/bar')
         self.repo.unlock_client('fooclient')
 
@@ -1717,7 +1746,6 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
             REPO_FILE_GROUPNAME,
             REPO_FILE_SYMLINK_TARGET,
             REPO_FILE_XATTR_BLOB,
-            REPO_FILE_MD5,
             ]
         for key in common:
             self.assertTrue(
@@ -1821,37 +1849,37 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
     # FIXME: These tests fails due to ClientMetadataTree brokenness, it seems.
     # They're disabled, for now. The bug is not exposed by existing code,
     # only by the new interface's tests.
-    if False:
-        def test_removing_file_removes_all_its_file_keys(self):
-            gen_id = self.create_generation()
-            self.repo.add_file(gen_id, '/foo/bar')
-            self.repo.set_file_key(
-                gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME, 123)
 
-            # Remove the file. Key should be removed.
-            self.repo.remove_file(gen_id, '/foo/bar')
-            self.assertRaises(
-                obnamlib.RepositoryFileDoesNotExistInGeneration,
-                self.repo.get_file_key,
-                gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME)
+    # def test_removing_file_removes_all_its_file_keys(self):
+    #     gen_id = self.create_generation()
+    #     self.repo.add_file(gen_id, '/foo/bar')
+    #     self.repo.set_file_key(
+    #         gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME_SEC, 123)
 
-            # Add the file back. Key should still be removed.
-            self.repo.add_file(gen_id, '/foo/bar')
-            value = self.repo.get_file_key(
-                gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME)
-            self.assertEqual(value, 0)
+    #     # Remove the file. Key should be removed.
+    #     self.repo.remove_file(gen_id, '/foo/bar')
+    #     self.assertRaises(
+    #         obnamlib.RepositoryFileDoesNotExistInGeneration,
+    #         self.repo.get_file_key,
+    #         gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME_SEC)
 
-        def test_can_add_a_file_then_remove_then_add_it_again(self):
-            gen_id = self.create_generation()
+    #     # Add the file back. Key should still be removed.
+    #     self.repo.add_file(gen_id, '/foo/bar')
+    #     value = self.repo.get_file_key(
+    #         gen_id, '/foo/bar', obnamlib.REPO_FILE_MTIME_SEC)
+    #     self.assertEqual(value, 0)
 
-            self.repo.add_file(gen_id, '/foo/bar')
-            self.assertTrue(self.repo.file_exists(gen_id, '/foo/bar'))
+    # def test_can_add_a_file_then_remove_then_add_it_again(self):
+    #     gen_id = self.create_generation()
 
-            self.repo.remove_file(gen_id, '/foo/bar')
-            self.assertFalse(self.repo.file_exists(gen_id, '/foo/bar'))
+    #     self.repo.add_file(gen_id, '/foo/bar')
+    #     self.assertTrue(self.repo.file_exists(gen_id, '/foo/bar'))
 
-            self.repo.add_file(gen_id, '/foo/bar')
-            self.assertTrue(self.repo.file_exists(gen_id, '/foo/bar'))
+    #     self.repo.remove_file(gen_id, '/foo/bar')
+    #     self.assertFalse(self.repo.file_exists(gen_id, '/foo/bar'))
+
+    #     self.repo.add_file(gen_id, '/foo/bar')
+    #     self.assertTrue(self.repo.file_exists(gen_id, '/foo/bar'))
 
     def test_unlocking_client_forgets_set_file_keys(self):
         gen_id = self.create_generation()
@@ -2108,7 +2136,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         token = self.repo.prepare_chunk_for_indexes('foochunk')
         self.repo.put_chunk_into_indexes(chunk_id, token, 'fooclient')
         self.assertEqual(
-            self.repo.find_chunk_ids_by_content('foochunk'), [chunk_id])
+            self.repo.find_chunk_ids_by_token(token), [chunk_id])
 
     def test_finds_all_matching_chunk_ids(self):
         self.setup_client()
@@ -2122,7 +2150,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.put_chunk_into_indexes(chunk_id_2, token, 'fooclient')
 
         self.assertEqual(
-            set(self.repo.find_chunk_ids_by_content('foochunk')),
+            set(self.repo.find_chunk_ids_by_token(token)),
             set([chunk_id_1, chunk_id_2]))
 
     def test_removes_chunk_from_indexes(self):
@@ -2134,7 +2162,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.remove_chunk_from_indexes(chunk_id, 'fooclient')
         self.assertRaises(
             obnamlib.RepositoryChunkContentNotInIndexes,
-            self.repo.find_chunk_ids_by_content, 'foochunk')
+            self.repo.find_chunk_ids_by_token, token)
 
     def test_removes_chunk_from_indexes_for_all_clients(self):
         self.setup_two_clients()
@@ -2146,7 +2174,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.remove_chunk_from_indexes_for_all_clients(chunk_id)
         self.assertRaises(
             obnamlib.RepositoryChunkContentNotInIndexes,
-            self.repo.find_chunk_ids_by_content, 'foochunk')
+            self.repo.find_chunk_ids_by_token, token)
 
     def test_putting_chunk_to_indexes_without_locking_them_fails(self):
         chunk_id = self.repo.put_chunk_content('foochunk')
@@ -2189,7 +2217,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.unlock_chunk_indexes()
         self.assertRaises(
             obnamlib.RepositoryChunkContentNotInIndexes,
-            self.repo.find_chunk_ids_by_content, 'foochunk')
+            self.repo.find_chunk_ids_by_token, token)
 
     def test_committing_chunk_indexes_remembers_changes(self):
         self.setup_client()
@@ -2200,7 +2228,7 @@ class RepositoryInterfaceTests(unittest.TestCase):  # pragma: no cover
         self.repo.commit_chunk_indexes()
         self.repo.unlock_chunk_indexes()
         self.assertEqual(
-            self.repo.find_chunk_ids_by_content('foochunk'), [chunk_id])
+            self.repo.find_chunk_ids_by_token(token), [chunk_id])
 
     def test_committing_does_NOT_remove_chunk_indexes_lock(self):
         self.repo.lock_chunk_indexes()
